$(document).ready(function() {


    if(window.widget)
    {
        // Refresh everytime the widget is shown.
        widget.onshow = refresh;
            
        // Save preferences when the widget is hidden.
        widget.onhide = save_preferences;
        
        // Remove preferences when the widget is removed.
        widget.onremove = remove_preferences;
    }


    // Show info on front hover.
    $('#front').livequery(function() {
        $(this).hover(function () {
            $('#info').fadeIn('fast');
            },
            function () {
            $('#info').fadeOut('fast');
        });
    });


    // Info.
    $('#info').livequery('click', function() {
        showBack();
    });


    // Done.
    $('#done').livequery('click', function() {
        setup();
    });


    // Start timer.
    $('.not_started').livequery('click', function() {
        $('#time').countdown({since: '-1S', compact: true, format: 'HMS'});
        $(this).addClass('pause').removeClass('not_started');
    });


    // Pause timer.
    $('.pause').livequery('click', function() {
        $('#time').countdown('pause');
        $(this).addClass('resume').removeClass('pause');
    });


    // Resumer timer.
    $('.resume').livequery('click', function() {
        $('#time').countdown('resume');
        $(this).addClass('pause').removeClass('resume');
    });


    // Reset timer.
    $('#reset').livequery('click', function() {
        $('#time').countdown('destroy').text('00:00:00'); 
        $('#control').addClass('not_started').removeClass('pause').removeClass('resume');	
    });


    // Remove success class on change.
    $('#machine_id, #api_token').livequery('change', function() {
        $('#done').removeClass('success');
    });


    // Add time.
    $('#add_time').livequery('click', function() {
        post();
    });

    
    // Help.
    $('#help').livequery('click', function() {
        widget.openURL('http://invoicemachine.com/help');
    });


    // Get update.
    $('#get_update').livequery('click', function() {
        widget.openURL('http://invoicemachine.com/extras');
    });
    

    // Remove required class.
    $('#projects').livequery('change', function() {
        if ($('#projects').val() != '') {
            $('#projects').removeClass('required');
        }
    });
    
    
    // Post.
    function post() {

        if ($('#projects').val() != '') {
        
            $('#indicator').fadeIn('slow');
            
            $('#projects').removeClass('required');
            
            $('#time').countdown('pause');
            
            $('#add_time').attr('disabled','disabled').val('Loading…');
            
            var machine_id = widget.preferenceForKey("machine_id");
            var api_token = widget.preferenceForKey("api_token");
            var url = 'http://' + machine_id + '.invoicemachine.com/api/';
            
            var id = $('select option:selected').val();
            var time = to_decimal($('#time').text());
            
            $.ajax({
                type: 'POST',
                url: url + 'project.time',
                data: 'api_token=' + api_token + '&id=' + id + '&time=' + time,
                success: function() {
                    $('#time').countdown('destroy').text('Added');
                    
                    refresh();
                    $('#control').addClass('not_started').removeClass('pause').removeClass('resume');
                    
                    
                    setTimeout(function() {
                        $('#time').text('00:00:00');
                        $('#add_time').removeAttr('disabled').removeClass('disabled').val('Add Time');
                        $('#indicator').fadeOut('slow');
                    }, 1000);
                    
                }
            });
            
        }
        else
        {

            $('#projects').addClass('required');
        }
        
    }


    // Remove preferences.
    function remove_preferences() {

        widget.setPreferenceForKey(null, 'machine_id');
        widget.setPreferenceForKey(null, 'api_token');
    }


    // Save preferences.
    function save_preferences() {

        widget.setPreferenceForKey($('#machine_id').val(), 'machine_id');
        widget.setPreferenceForKey($('#api_token').val(), 'api_token');
    }


    // Refresh.
    function refresh() {
    
        // Check for update.
        update();

        var machine_id = widget.preferenceForKey("machine_id");
        var api_token = widget.preferenceForKey("api_token");
        var url = 'http://' + machine_id + '.invoicemachine.com/api/';
        var id = $('select option:selected').val();
        
        $('#machine_id').val(machine_id);
        $('#api_token').val(api_token);
        
        $('#indicator').fadeIn('slow');
        
        $.ajax({
            type: 'POST',
            url: url + 'project.getall',
            data: 'api_token=' + api_token,
            success: function(data) {
            
                append(data);
                
                $('#indicator').fadeOut('slow');
                $('#done').addClass('success');
                
                $('#projects').val(id).attr('selected');
            },
            error: function(){
                $('#indicator').fadeOut('slow');
                
                if ($('#back').is(':hidden')) {
                    showBack();
                }
            }
        });
    }

    
    // Append.
    function append(data) {
        $('#projects').empty().append('<option value=""></option');
        $('project',data).each(function() {
        
            var hour = remove_zeros($('hours',this).text());
        
            if (hour < '2') {
                var hours = ' (' + hour + ' hour)';
            }
            else {
                var hours = ' (' + hour + ' hours)';
            }
        
            $('#projects').append('<option value="' + $('id',this).text() + '">' + $('description',this).text() + hours + '</option>');
        });
    }


    // Setup.
    function setup() {

        save_preferences();

        var machine_id = widget.preferenceForKey("machine_id");
        var api_token = widget.preferenceForKey("api_token");
        var url = 'http://' + machine_id + '.invoicemachine.com/api/';
        
        $('#status').removeClass('error').empty().append('Checking…').fadeIn('fast');

        if (machine_id == '' || api_token == '') {
            $('#status').addClass('error').empty().append('Enter your Machine ID and API Token.').fadeIn('fast');
        }
        else {
            
            if ($('#done').hasClass('success')) {
                showFront();
                $('#status').hide();
                $('#indicator').fadeOut('slow');
            }
            else {
                $.ajax({
                    type: 'POST',
                    url: url + 'project.getall',
                    data: 'api_token=' + api_token,
                    success: function(data) {
                    
                        append(data);
                    
                        showFront();
                        $('#indicator').fadeIn('slow');
                        
                        $('#status').hide();
                        $('#done').addClass('success');
                        
                        setTimeout(function() {
                            $('#indicator').fadeOut('slow');
                        }, 1000);
                        
                    },
                    error: function(){
                        $('#status').addClass('error').empty().append('The Machine ID or API token is invalid.').fadeIn('fast');
                    }
                });
            }
        }
    }
    

    // Update.
    function update() {
        var url = 'http://update.invoicemachine.com/api/';
        
        $.ajax({
            type: 'POST',
            url: url + 'project.update',
            success: function(data) {
            
                if (data == 'true') {
                    $('#preferences').hide();
                    $('#update').show();
                    if ($('#back').is(':hidden')) {
                        showBack();
                    }
                }
            }
        });
    }


    // Apple's function to show the back of the widget.
    function showBack(event)
    {
        var front = document.getElementById("front");
        var back = document.getElementById("back");

        if (window.widget) {
            widget.prepareForTransition("ToBack");
        }

        front.style.display = "none";
        back.style.display = "block";

        if (window.widget) {
            setTimeout('widget.performTransition();', 0);
        }
    }


    // Apple's function to show the front of the widget.
    function showFront(event)
    {
        var front = document.getElementById("front");
        var back = document.getElementById("back");

        if (window.widget) {
            widget.prepareForTransition("ToFront");
        }

        front.style.display="block";
        back.style.display="none";

        if (window.widget) {
            setTimeout('widget.performTransition();', 0);
        }
    }


    // Function to convert time to decimal format.
    function to_decimal(time) {
        var split = time.split(':');
        var hour = parseFloat(split[0]);
        var min = parseFloat(split[1]);
        var sec = parseFloat(split[2]);
        return (hour + min/60 + sec/3600).toFixed(2);
    }

    
    // Function to remove unnecessary zeros from time.
	function remove_zeros(time) {
    
        if (time == '0.00') {
            return time;
        }
        else {
            var decPos = time.indexOf('.')
            if (decPos>-1) {
                first = time.substring(0,decPos);
                second = time.substring(decPos,time.length);
                while (second.charAt(second.length-1) == '0')
                second=second.substring(0,second.length-1);
                if (second.length>1)
                return first+second;
                else
                return first;
            }
            return time;
        }
	}


});